package git

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Reference.PSource

Sourcetype partial

Type of the left part of a reference.

Sourcetype branch = string

Tyoe of the right part of a reference.

Sourceval (//) : partial -> string -> partial

Infix operator to compose string value with a partial reference.

Sourceval (/) : partial -> branch -> t

Infix operator to compose partial with branch and return a full-defined reference t.

Sourceval refs : partial

refs/

Sourceval heads : partial

refs/heads/

Sourceval remotes : partial

refs/remotes/

Sourceval origin : partial

refs/remotes/origin

Sourceval master : branch

*/master