package git

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Git.SmartSource

Sourcemodule type COMMON = sig ... end
Sourcemodule Common (Hash : sig ... end) (Reference : Reference.S with module Hash := Hash) : COMMON with type hash := Hash.t and type reference := Reference.t

The Smart protocol including the encoder and the decoder.

Sourcemodule type DECODER = sig ... end
Sourcemodule Decoder (Hash : sig ... end) (Reference : Reference.S with module Hash := Hash) (Common : COMMON with type hash := Hash.t and type reference := Reference.t) : DECODER with module Hash := Hash and module Reference := Reference and module Common := Common

The functor to make the Decoder by a specific hash implementation.

Sourcemodule type ENCODER = sig ... end
Sourcemodule Encoder (Hash : sig ... end) (Reference : Reference.S with module Hash := Hash) (Common : COMMON with type hash := Hash.t and type reference := Reference.t) : ENCODER with module Hash := Hash and module Reference := Reference and module Common := Common

The functor to make the Encoder by a specific hash implementation.

Sourcemodule type CLIENT = sig ... end
Sourcemodule Client (Hash : sig ... end) (Reference : Reference.S with module Hash := Hash) : CLIENT with module Hash := Hash and module Reference := Reference

The functor to make the Client by a specific hash implementation.