package git

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Tag.MakeSource

The functor to make the OCaml representation of the Git Tag object by a specific hash implementation.

Parameters

module Hash : sig ... end

Signature

Sourcetype t

A Git Tag object. The tag object is very much like a Commit.t object - it contains a tagger, a date, a message, and a pointer. Generally, the tag points to a commit rather than a tree. It's like a branch reference, but it never moves - it always points to the same commit but gives it a friendlier name.

Sourcetype kind =
  1. | Blob
  2. | Commit
  3. | Tag
  4. | Tree
Sourceval make : Hash.t -> kind -> ?tagger:User.t -> tag:string -> string -> t

make hash kind ?tagger ~tag descr makes a new tag to a Git object with the kind kind by the tagger with the name tag and the description descr.

This function does not check if the Git object pointed by the hash has really the kind kind - and obviously if the hash is valid.

Sourcemodule MakeMeta (Meta : Encore.Meta.S) : sig ... end
Sourcemodule A : sig ... end
Sourcemodule M : sig ... end
Sourcemodule D : sig ... end
Sourcemodule E : sig ... end
Sourceval digest : t -> Hash.t
Sourceval pp : t Fmt.t
Sourceval compare : t -> t -> int
Sourceval hash : t -> int
Sourceval equal : t -> t -> bool
Sourcemodule Set : Set.S with type elt = t
Sourcemodule Map : Map.S with type key = t
Sourceval length : t -> int64

length t returns the length of the tag object t.

Sourceval obj : t -> Hash.t

obj t returns the pointed hash of the Tag t.

Sourceval tag : t -> string

tag t returns the tag information of t.

Sourceval message : t -> string
Sourceval kind : t -> kind
Sourceval tagger : t -> User.t option